@@ -34,12 +34,11 @@
 
 #define APPNAME "tagidx"
 
-#include "TagidxOptions.h"
+#include "TagidxParser.h"
 
-#include <tagcoll/BasicStringDiskIndex.h>
-#include <tagcoll/ItemGrouper.h>
-#include <tagcoll/Consumer.h>
-#include <tagcoll/PatchCollection.h>
+#include <BasicStringDiskIndex.h>
+#include <tagcoll/coll/fast.h>
+#include <tagcoll/coll/patched.h>
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -50,7 +49,7 @@
 #include <stdlib.h>	// getenv
 #include <errno.h>
 
-#include <tagcoll/StdioParserInput.h>
+#include <tagcoll/input/stdio.h>
 #include <tagcoll/TextFormat.h>
 
 #include <iostream>
@@ -69,7 +68,7 @@
 #endif
 
 using namespace std;
-using namespace Tagcoll;
+using namespace tagcoll;
 
 string get_username()
 {
@@ -97,11 +96,12 @@ void mkpath(const std::string& dir)
 		if (errno == ENOENT)
 		{
 			if (mkdir(dir.c_str(), 0777) == -1)
-				throw SystemException(errno, "creating directory " + dir);
+				throw wibble::exception::System("creating directory " + dir);
 		} else
-			throw SystemException(errno, "getting informations about " + dir);
+			throw wibble::exception::System("getting informations about " + dir);
 	} else if (!S_ISDIR(st.st_mode))
-		throw ConsistencyCheckException("ensuring that " + dir + " is a directory");
+		throw wibble::exception::Consistency("performing sanity checks on path components",
+				dir + " is not a directory");
 }
 
 
@@ -147,20 +147,18 @@ OpSet<string> getItems(HierarchyNode<string, string>* node)
 }
 #endif
 
-void readCollection(const string& file, Consumer<string, string>& builder)
-	throw (FileException, ParserException)
+template<typename OUT>
+void readCollection(const string& file, const OUT& out)
 {
-	TrivialConverter<string, string> conv;
-
 	if (file == "-")
 	{
-		StdioParserInput input(stdin, "<stdin>");
-		TextFormat<string, string>::parse(conv, conv, input, builder);
+		input::Stdio input(stdin, "<stdin>");
+		textformat::parse(input, out);
 	}
 	else
 	{
-		StdioParserInput input(file);
-		TextFormat<string, string>::parse(conv, conv, input, builder);
+		input::Stdio input(file);
+		textformat::parse(input, out);
 	}
 }
 
@@ -182,7 +180,7 @@ PatchList<string, string> readPatchesFromDir(const std::string& dirname)
 
 	DIR* dir = opendir(dirname.c_str());
 	if (dir == NULL)
-		throw SystemException(errno, "opening directory " + dirname);
+		throw wibble::exception::System("opening directory " + dirname);
 	
 	struct dirent* ent;
 	while ((ent = readdir(dir)) != NULL)
@@ -194,7 +192,7 @@ PatchList<string, string> readPatchesFromDir(const std::string& dirname)
 		string file = dirname + "/" + ent->d_name;
 		struct stat st;
 		if (stat(file.c_str(), &st) == -1)
-			throw SystemException(errno, "getting informations about " + file);
+			throw wibble::exception::System("getting informations about " + file);
 
 		// We only want regular files or symlinks
 		if (!(S_ISREG(st.st_mode) || S_ISLNK(st.st_mode)))
@@ -211,7 +209,7 @@ void removePatchesFromDir(const std::string& dirname)
 {
 	DIR* dir = opendir(dirname.c_str());
 	if (dir == NULL)
-		throw SystemException(errno, "opening directory " + dirname);
+		throw wibble::exception::System("opening directory " + dirname);
 	
 	struct dirent* ent;
 	while ((ent = readdir(dir)) != NULL)
@@ -223,14 +221,14 @@ void removePatchesFromDir(const std::string& dirname)
 		string file = dirname + "/" + ent->d_name;
 		struct stat st;
 		if (stat(file.c_str(), &st) == -1)
-			throw SystemException(errno, "getting informations about " + file);
+			throw wibble::exception::System("getting informations about " + file);
 
 		// We only want regular files or symlinks
 		if (!(S_ISREG(st.st_mode) || S_ISLNK(st.st_mode)))
 			continue;
 
 		if (unlink(file.c_str()) == -1)
-			throw SystemException(errno, "removing file " + file);
+			throw wibble::exception::System("removing file " + file);
 	}
 	closedir(dir);
 }
@@ -258,11 +256,11 @@ public:
 		
 		int fd = open(patchfile.c_str(), O_CREAT | O_WRONLY | (overwrite ? O_TRUNC : O_EXCL), 0666);
 		if (fd == -1)
-			throw SystemException(errno, "creating file " + patchfile);
+			throw wibble::exception::System("creating file " + patchfile);
 
 		FILE* out = fdopen(fd, "w");
 		if (out == NULL)
-			throw SystemException(errno, "fdopening file " + patchfile);
+			throw wibble::exception::System("fdopening file " + patchfile);
 
 		TextFormat<string, string>::outputPatch(conv, conv, patch, out);
 		fclose(out);
@@ -279,7 +277,7 @@ public:
 			if (errno == ENOTEMPTY)
 				cerr << "Directory " + patchdir + " was not empty: skipping removing it";
 			else
-				throw SystemException(errno, "Removing directory " + patchdir);
+				throw wibble::exception::System("Removing directory " + patchdir);
 
 		unlink(file.c_str());
 	}
@@ -552,9 +550,9 @@ public:
 };
 #endif
 
-void output(commandline::TagidxOptions& opts, ReadonlyCollection<string, string>& coll)
+void output(wibble::commandline::TagidxParser& opts, ReadonlyCollection<string, string>& coll)
 {
-	if (opts.outputGroup.quiet->boolValue())
+	if (opts.out_quiet->boolValue())
 	{
 		Sink<string, string> sink;
 		coll.output(sink);
@@ -562,7 +560,7 @@ void output(commandline::TagidxOptions& opts, ReadonlyCollection<string, string>
 		TrivialConverter<string, string> conv;
 		TextFormat<string, string> writer(conv, conv, stdout);
 
-		if (opts.outputGroup.group->boolValue())
+		if (opts.out_group->boolValue())
 		{
 			ItemGrouper<string, string> grouper;
 			coll.output(grouper);
@@ -572,66 +570,32 @@ void output(commandline::TagidxOptions& opts, ReadonlyCollection<string, string>
 	}
 }
 
-string indexname(commandline::TagidxOptions& opts)
+string indexname(wibble::commandline::TagidxParser& opts)
 {
-	if (opts.indexGroup.index->boolValue())
-		return opts.indexGroup.index->stringValue();
+	if (opts.idx_index->boolValue())
+		return opts.idx_index->stringValue();
 	if (getenv("TAGIDX") != NULL)
 		return getenv("TAGIDX");
 	struct passwd* pw = getpwuid(getuid());
 	if (pw != NULL)
 		return string(pw->pw_dir) + "/.tagcoll/index";
-	throw NotFoundException("looking for a suitable place for the index directory (tried the --index commandline option, the $TAGIDX environment variable and the home directory)");
+	throw wibble::exception::Consistency("computing the pathname for the index directory",
+			"cannot find a suitable place for the index directory"
+			"(tried the --index commandline option, "
+			"the $TAGIDX environment variable and the home directory)");
 }
 
 int main(int argc, const char* argv[])
 {
-	commandline::TagidxOptions opts;
+	wibble::commandline::TagidxParser opts;
 
 	try {
-		opts.parse(argc, argv);
-		if (!opts.lastCommand())
-			throw commandline::BadOption("could not understand the command to execute");
-		
-		if (opts.helpGroup.help->boolValue())
-		{
-			// Provide help as requested
-			commandline::Help help(APPNAME, VERSION);
-			commandline::OptionParser* o = opts.lastCommand();
+		// Parse commandline.
+		// If the parser took care of the action, we can exit just fine.
+		if (opts.parse(argc, argv))
+			return 0;
 
-			if (o && !o->name().empty())
-				// Help on a specific command
-				help.outputHelp(cout, *o);
-			else
-				// General help
-				help.outputHelp(cout, opts);
-		}
-		else if (opts.helpGroup.version->boolValue())
-		{
-			// Print the program version
-			commandline::Help help(APPNAME, VERSION);
-			help.outputVersion(cout);
-		}
-		else if (opts.lastCommand() == &opts.generic)
-		{
-			commandline::Help help(APPNAME, VERSION);
-			help.outputHelp(cout, opts);
-		}
-		else if (opts.lastCommand() == &opts.help)
-		{
-			commandline::Help help(APPNAME, VERSION);
-			commandline::OptionParser* o = 0;
-			if (opts.hasNext())
-				o = opts.command(opts.next());
-
-			if (o)
-				// Help on a specific command
-				help.outputHelp(cout, *o);
-			else
-				// General help
-				help.outputHelp(cout, opts);
-		}
-		else if (opts.lastCommand() == &opts.create)
+		if (opts.foundCommand() == opts.cmd_create)
 		{
 			string fname = indexname(opts);
 
@@ -645,12 +609,12 @@ int main(int argc, const char* argv[])
 
 			indexer.write(fname);
 		}
-		else if (opts.lastCommand() == &opts.remove)
+		else if (opts.foundCommand() == opts.cmd_remove)
 		{
 			string fname = indexname(opts);
 			Index::remove(fname);
 		}
-		else if (opts.lastCommand() == &opts.addpatch)
+		else if (opts.foundCommand() == opts.cmd_addpatch)
 		{
 			string fname = indexname(opts);
 			Index idx(fname);
@@ -666,7 +630,7 @@ int main(int argc, const char* argv[])
 					PatchList<string, string> patch = readPatchFromFile(filename);
 					if (patch.size() > 0)
 					{
-						idx.addPatch(patch, name, opts.addpatch.force->boolValue());
+						idx.addPatch(patch, name, opts.misc_force->boolValue());
 					} else {
 						cerr << "skipped empty patch file " << filename << endl;
 					}
@@ -679,7 +643,7 @@ int main(int argc, const char* argv[])
 				addPatch(idx, patch);
 			}
 		}
-		else if (opts.lastCommand() == &opts.compact)
+		else if (opts.foundCommand() == opts.cmd_compact)
 		{
 			string fname = indexname(opts);
 
@@ -697,7 +661,7 @@ int main(int argc, const char* argv[])
 			// Write the new index
 			indexer.write(fname);
 		}
-		else if (opts.lastCommand() == &opts.cat)
+		else if (opts.foundCommand() == opts.cmd_cat)
 		{
 			string fname = indexname(opts);
 			Index idx(fname);
@@ -705,23 +669,16 @@ int main(int argc, const char* argv[])
 			output(opts, idx);
 		}
 		else
-			throw commandline::BadOption(string("unhandled command ") +
-						(opts.lastCommand() ? opts.lastCommand()->name() : "(null)"));
+			throw wibble::exception::BadOption(string("unhandled command ") +
+						(opts.foundCommand() ? opts.foundCommand()->name() : "(null)"));
 
 		return 0;
-	} catch (commandline::BadOption& e) {
+	} catch (wibble::exception::BadOption& e) {
 		cerr << e.desc() << endl;
-		commandline::Help help("tagidx", VERSION);
-		if (opts.lastCommand())
-		{
-			help.outputHelp(cerr, *opts.lastCommand());
-		} else {
-			help.outputHelp(cerr, opts);
-		}
-		exit(1);
-	} catch (Exception& e)
-	{
-		fprintf(stderr, "%s: %.*s\n", e.type(), PFSTR(e.desc()));
+		opts.outputHelp(cerr);
+		return 1;
+	} catch (std::exception& e) {
+		cerr << e.what() << endl;
 		return 1;
 	}
 }
